package sun.nio.cs;

import java.util.ArrayList;
import java.util.Iterator;
import java.util.TreeMap;
import sun.misc.ASCIICaseInsensitiveComparator;
import java.nio.charset.Charset;
import java.lang.ref.SoftReference;
import java.util.Map;
import java.nio.charset.spi.CharsetProvider;

public class AbstractCharsetProvider extends CharsetProvider
{
    private Map<String, String> classMap;
    private Map<String, String> aliasMap;
    private Map<String, String[]> aliasNameMap;
    private Map<String, SoftReference<Charset>> cache;
    private String packagePrefix;
    
    protected AbstractCharsetProvider() {
        this.classMap = new TreeMap(ASCIICaseInsensitiveComparator.CASE_INSENSITIVE_ORDER);
        this.aliasMap = new TreeMap(ASCIICaseInsensitiveComparator.CASE_INSENSITIVE_ORDER);
        this.aliasNameMap = new TreeMap(ASCIICaseInsensitiveComparator.CASE_INSENSITIVE_ORDER);
        this.cache = new TreeMap(ASCIICaseInsensitiveComparator.CASE_INSENSITIVE_ORDER);
        this.packagePrefix = "sun.nio.cs";
    }
    
    protected AbstractCharsetProvider(String packagePrefix) {
        this.classMap = new TreeMap(ASCIICaseInsensitiveComparator.CASE_INSENSITIVE_ORDER);
        this.aliasMap = new TreeMap(ASCIICaseInsensitiveComparator.CASE_INSENSITIVE_ORDER);
        this.aliasNameMap = new TreeMap(ASCIICaseInsensitiveComparator.CASE_INSENSITIVE_ORDER);
        this.cache = new TreeMap(ASCIICaseInsensitiveComparator.CASE_INSENSITIVE_ORDER);
        this.packagePrefix = packagePrefix;
    }
    
    private static <K, V> void put(Map<K, V> map, K k, V v) {
        if (!map.containsKey(k)) {
            map.put(k, v);
        }
    }
    
    private static <K, V> void remove(Map<K, V> map, K k) {
        Object remove = map.remove(k);
        assert remove != null;
        return;
    }
    
    protected void charset(String s, String s2, String[] array) {
        synchronized (this) {
            put(this.classMap, s, s2);
            for (int i = 0; i < array.length; ++i) {
                put(this.aliasMap, array[i], s);
            }
            put(this.aliasNameMap, s, array);
            this.cache.clear();
        }
    }
    
    protected void deleteCharset(String s, String[] array) {
        synchronized (this) {
            remove(this.classMap, s);
            for (int i = 0; i < array.length; ++i) {
                remove(this.aliasMap, array[i]);
            }
            remove(this.aliasNameMap, s);
            this.cache.clear();
        }
    }
    
    protected void init() {
    }
    
    private String canonicalize(String s) {
        String s2 = (String)this.aliasMap.get(s);
        return (s2 != null) ? s2 : s;
    }
    
    private Charset lookup(String s) {
        SoftReference<Object> softReference = (SoftReference)this.cache.get(s);
        if (softReference != null) {
            Charset charset = (Charset)softReference.get();
            if (charset != null) {
                return charset;
            }
        }
        String str = (String)this.classMap.get(s);
        if (str == null) {
            return null;
        }
        try {
            Charset referent = (Charset)Class.forName(this.packagePrefix + "." + str, true, this.getClass().getClassLoader()).newInstance();
            this.cache.put(s, new SoftReference(referent));
            return referent;
        }
        catch (ClassNotFoundException ex) {
            return null;
        }
        catch (IllegalAccessException ex2) {
            return null;
        }
        catch (InstantiationException ex3) {
            return null;
        }
    }
    
    @Override
    public final Charset charsetForName(String s) {
        synchronized (this) {
            this.init();
            return this.lookup(this.canonicalize(s));
        }
    }
    
    @Override
    public final Iterator<Charset> charsets() {
        final ArrayList list;
        synchronized (this) {
            this.init();
            list = new ArrayList(this.classMap.keySet());
        }
        return new Iterator<Charset>() {
            Iterator<String> i = list.iterator();
            
            @Override
            public boolean hasNext() {
                return this.i.hasNext();
            }
            
            @Override
            public Charset next() {
                String s = (String)this.i.next();
                synchronized (AbstractCharsetProvider.this) {
                    return AbstractCharsetProvider.access$000(AbstractCharsetProvider.this, s);
                }
            }
            
            @Override
            public void remove() {
                throw new UnsupportedOperationException();
            }
        };
    }
    
    public final String[] aliases(String s) {
        synchronized (this) {
            this.init();
            return (String[])this.aliasNameMap.get(s);
        }
    }
    
    static Charset access$000(AbstractCharsetProvider abstractCharsetProvider, String s) {
        return abstractCharsetProvider.lookup(s);
    }
}
